;; 
;; qwiki-post-commit-hook - updater script for the quick wiki
;; 
;; Copyright (c) 2009-2010 Peter Bex and Ivan Raikov
;;
;;  Redistribution and use in source and binary forms, with or without
;;  modification, are permitted provided that the following conditions
;;  are met:
;; 
;;  - Redistributions of source code must retain the above copyright
;;  notice, this list of conditions and the following disclaimer.
;; 
;;  - Redistributions in binary form must reproduce the above
;;  copyright notice, this list of conditions and the following
;;  disclaimer in the documentation and/or other materials provided
;;  with the distribution.
;; 
;;  - Neither name of the copyright holders nor the names of its
;;  contributors may be used to endorse or promote products derived
;;  from this software without specific prior written permission.
;; 
;;  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND THE
;;  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
;;  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;;  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;;  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR THE
;;  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
;;  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
;;  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
;;  USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
;;  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
;;  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
;;  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
;;  POSSIBILITY OF SUCH DAMAGE.

(module qwiki-post-commit-hook
  (qwiki-post-commit-hook!)

(import scheme chicken data-structures files)
(use posix svn-client qwiki-svn qwiki-sxml qwiki)

;; TODO:
;; * How do we clean up directories in the html cache? This is a problem
;;    when someone removes a dir and then creates a file to replace it.
;; * This sometimes updates files twice or more: This has something to
;;    to with the fact that creating a new revision of a file causes
;;    that file's version to be bumped, but when the toplevel dir
;;    is updated, we're still looking at the history from the the
;;    toplevel dir's current revision up to the head revision.
;; * How to handle locking? The checkout could be locked when someone is
;;    performing a local commit at the same time someone else is editing
;;    the wiki through the web interface.
(define (qwiki-post-commit-hook!)
  (and-let* (((directory? (qwiki-source-path)))
             (info-list '())
             (info (svn-client-info (qwiki-source-path)
                                    svn-opt-revision-unspecified
                                    svn-opt-revision-unspecified
                                    (lambda (path i)
                                      (set! info-list (cons i info-list)))
                                    #f
                                    (qwiki-repos-username)
                                    (qwiki-repos-password)))
             ((pair? info-list))
             (revision (svn-info-rev (car info-list))))
    ;; First, get the files up to date. We use this as a guide to see what
    ;; to do with the generated information. If the file is missing, we can
    ;; delete it. If was added or modified it will exist, and we can update it.
    (update-sources! (qwiki-source-path))
    (svn-client-log
     (qwiki-repos-uri) (make-svn-opt-revision-number revision)
     svn-opt-revision-head #t #f
     (qwiki-repos-username) (qwiki-repos-password)
     (lambda (paths . rest)
       (for-each
        (lambda (path/changed-path-pair)
          (let* ((path (car path/changed-path-pair))
                 (source-path (make-pathname (qwiki-source-path) path)))
            (if (file-exists? source-path)
                (unless (symbolic-link? source-path)
                  ;; Assume the VCS backend won't hand us paths with non-leaf
                  ;; symlink components!
                  (qwiki-update-file! (string-split path "/")))
                (qwiki-delete-file! (string-split path "/")))))
        paths)))))

); end module
